import {
  select,
  csv,
  scaleLinear,
  scaleTime,
  scaleOrdinal,
  extent,
  axisLeft,
  axisBottom,
  line,
  curveBasis,
  nest,
  timeParse,
  schemeCategory10,
  descending,
  format,
} from 'd3';

import { colorLegend } from './colorLegend';

const svg = select('svg');

const width = +svg.attr('width');
const height = +svg.attr('height');

const render = (data) => {
  const title = 'Military Expenditure % of GDP';

  const xValue = (d) => d.year;
  const xAxisLabel = 'Time';

  const yValue = (d) => d.spending;
  const circleRadius = 6;
  const yAxisLabel = 'Spending';

  const colorValue = (d) => d.country;

  const margin = {
    top: 60,
    right: 280,
    bottom: 88,
    left: 105,
  };
  const innerWidth =
    width - margin.left - margin.right;
  const innerHeight =
    height - margin.top - margin.bottom;

  const xScale = scaleTime()
    .domain(extent(data, xValue))
    .range([0, innerWidth]);

  const yScale = scaleLinear()
    .domain(extent(data, yValue))
    .range([innerHeight, 0])
    .nice();

  const colorScale = scaleOrdinal(
    schemeCategory10
  );

  const g = svg
    .append('g')
    .attr(
      'transform',
      `translate(${margin.left},${margin.top})`
    );

  const xAxis = axisBottom(xScale)
    .tickSize(-innerHeight)
    .tickPadding(15);

  /*const yAxisTickFormat = (number) =>
    format('.2s')(number)
      .replace('G', 'B')
      .replace('.0', '');
*/
  const yAxis = axisLeft(yScale)
    .tickSize(-innerWidth)
  //  .tickFormat(yAxisTickFormat)
    .tickPadding(10);

  const yAxisG = g.append('g').call(yAxis);
  yAxisG.selectAll('.domain').remove();

  yAxisG
    .append('text')
    .attr('class', 'axis-label')
    .attr('y', -60)
    .attr('x', -innerHeight / 2)
    .attr('fill', 'black')
    .attr('transform', `rotate(-90)`)
    .attr('text-anchor', 'middle')
    .text(yAxisLabel);

  const xAxisG = g
    .append('g')
    .call(xAxis)
    .attr(
      'transform',
      `translate(0,${innerHeight})`
    );

  xAxisG.select('.domain').remove();

  xAxisG
    .append('text')
    .attr('class', 'axis-label')
    .attr('y', 60)
    .attr('x', innerWidth / 2)
    .attr('fill', 'black')
    .text(xAxisLabel);

  const lineGenerator = line()
    .x((d) => xScale(xValue(d)))
    .y((d) => yScale(yValue(d)));
   // .curve(curveBasis);

  const lastYValue = (d) =>
    yValue(d.values[d.values.length - 1]);

  const nested = nest()
    .key(colorValue)
    .entries(data)
    .sort((a, b) =>
      descending(lastYValue(a), lastYValue(b))
    );

  colorScale.domain(nested.map((d) => d.key));

  g.selectAll('.line-path')
    .data(nested)
    .enter()
    .append('path')
    .attr('class', 'line-path')
    .attr('d', (d) => lineGenerator(d.values))
    .attr('stroke', (d) => colorScale(d.key));

  g.append('text')
    .attr('class', 'title')
    .attr('y', -10)
    .text(title);

  svg
    .append('g')
    .attr('transform', `translate(700,110)`)
    .call(colorLegend, {
      colorScale,
      circleRadius: 5,
      spacing: 15,
      textOffset: 10,
    });
};
csv('data.csv').then((data) => {
  data.forEach((d) => {
    d.year = new Date(d.year + ' ');
    d.spending = +d.spending;
  });
  render(data);
});
//loadAndProcessData().then(render);
